import { ChartDataset } from '../index.esm';

const dataset: ChartDataset = {
  data: [10, null, 20],
};

const lineDataset: ChartDataset<'line'> = {
  data: [10, null, 20],
};
const scatterDataset: ChartDataset<'scatter'> = {
  data: [10, null, 20],
};
const radarDataset: ChartDataset<'radar'> = {
  data: [10, null, 20],
};

