/**
 * Format a number using a localized number formatter.
 * @param num The number to format
 * @param locale The locale to pass to the Intl.NumberFormat constructor
 * @param options Number format options
 */
export function formatNumber(num: number, locale: string, options: Intl.NumberFormatOptions): string;
